extends Node2D


var Obstacle = preload("res://Scenes/Obstacle.tscn")
var Obstacle2 = preload("res://Scenes/JumpingObstacle.tscn")
var Card = preload("res://Scenes/Card.tscn")

export var spawn_offset = 5

onready var worldOne = get_node("Backgrounds/ParallaxLayer/WorldOne")
onready var worldTwo = get_node("Backgrounds/ParallaxLayer/WorldTwo")
onready var worldThree = get_node("Backgrounds/ParallaxLayer/WorldThree")
onready var worldFour = get_node("Backgrounds/ParallaxLayer/WorldFour")
onready var worldFive = get_node("Backgrounds/ParallaxLayer/WorldFive")

onready var obstacleSpawnPointBottom = get_node("ObstacleSpawnPointBottom")
onready var obstacleSpawnPointTop = get_node("ObstacleSpawnPointTop")

onready var boyLifePointOne = get_node("LifePoints/BoyLifePointOne")
onready var boyLifePointTwo = get_node("LifePoints/BoyLifePointTwo")
onready var boyLifePointThree = get_node("LifePoints/BoyLifePointThree")

onready var girlLifePointOne = get_node("LifePoints/GirlLifePointOne")
onready var girlLifePointTwo = get_node("LifePoints/GirlLifePointTwo")
onready var girlLifePointThree = get_node("LifePoints/GirlLifePointThree")

onready var player = get_node("Player")
onready var tween = player.tween

onready var cardSpawn = get_node("CardSpawn")
onready var cardCollectedPoint = get_node("CardCollectedPoint")

var lifePoints = 3
var spawn_timer = 0
var rng = RandomNumberGenerator.new()
var score = 0

var worldChooser
var characterChooser

var current_card = Card.instance()


# Called when the node enters the scene tree for the first time.
func _ready():
	OpenWorldChooser()
	OpenCharacterChooser()
	add_child(current_card)
	current_card.global_position = cardSpawn.global_position
	current_card.card_collider_detection.disabled = false
	current_card.PickRandomCard()


# Called every frame. 'delta' is the elapsed time since the previous frame.
func _process(delta):
	spawn_timer += delta
	
	if spawn_timer >= spawn_offset: # spawn a new card and obstacle
		var new_obstacle
		var obstacle_type = rng.randi_range(1, 2)
		
		if obstacle_type == 1:
			new_obstacle = Obstacle.instance()
		elif obstacle_type == 2:
			new_obstacle = Obstacle2.instance()
		
		if rng.randi_range(0,1) == 0:
			new_obstacle.global_position = obstacleSpawnPointBottom.global_position
		else:
			new_obstacle.global_position = obstacleSpawnPointTop.global_position
			
		add_child(new_obstacle)
		
		if current_card.card_entered_screen == false:
			current_card.global_position = cardSpawn.global_position
			current_card.card_collider_detection.disabled = false
			current_card.PickRandomCard()
		
		spawn_timer = 0 # reset timer
	
	if current_card.get_position().x < 0 and current_card.active_card != null:
		current_card.active_card.visible = false


# This method will increase the score by every card that has been collected.
func ChangeScore():
	score += 1
	var scoreLabel = get_node("Score")
	scoreLabel.set_text("Karten: " + str(score) + "/16")
	current_card.global_position = cardCollectedPoint.global_position
	current_card.RemoveCardWhichWasCollected()
	
	if score >= 16:
		var youWon = load("res://Scenes/YouWon.tscn").instance()
		get_parent().add_child(youWon)
		player.touch_buttons.get_child(0).hide()
		player.touch_buttons.get_child(1).hide()
		player.touch_buttons.get_child(2).hide()
		player.touch_buttons.get_child(3).hide()
		get_tree().paused = true # pause game in background
		


# This function will decrease the live points of the player.
func DecreaseLivePoints():
	lifePoints -= 1
	
	# check which symbol to delete
	if lifePoints == 2:
		boyLifePointThree.queue_free()
		girlLifePointThree.queue_free()
	elif lifePoints == 1:
		boyLifePointTwo.queue_free()
		girlLifePointTwo.queue_free()
	elif lifePoints == 0:
		boyLifePointOne.queue_free()
		girlLifePointOne.queue_free()
	
	DamageEffectBlinking()
	# check if the game if over
	CheckIfGameOver()


# When the player gets hit by a obstacle create blinking effect.
func DamageEffectBlinking():
	for number in 3:
		if player.animation_boy.visible == true:
			tween.interpolate_property($Player/Boy, "modulate:a", 1, 0, 0.3)
		else:
			tween.interpolate_property($Player/Girl, "modulate:a", 1, 0, 0.3)
		tween.start()
		yield(tween, "tween_completed")
		if player.animation_boy.visible == true:
			tween.interpolate_property($Player/Boy, "modulate:a", 0, 1, 0.1)
		else:
			tween.interpolate_property($Player/Girl, "modulate:a", 0, 1, 0.1)
		tween.start()
		yield(tween, "tween_completed")


# Check if the player has lost the game.
func CheckIfGameOver():
	# happens when the live points shrink to 0
	if(lifePoints <= 0):
		get_tree().paused = true # pause game in background
		ResetWorldsAtGameOver()
		var gameOver = load("res://Scenes/GameOver.tscn").instance()
		current_card.game_over_instance = gameOver
		get_parent().add_child(gameOver)
		current_card.PrepareCardsWhichWereCollected()
		gameOver.ShowCollectedCard()
		gameOver.pointsOfPlayer = score
		gameOver.ShowPoints()


# Open the screen to choose a world.
func OpenWorldChooser():
	worldChooser = load("res://Scenes/WorldChooser.tscn").instance()
	worldChooser.world = self
	get_parent().add_child(worldChooser)


# Load first world and set font color.
func LoadWorldOne():
	SetFontColor(Color.black)
	worldOne.visible = true


# Load second world and set font color.
func LoadWorldTwo():
	SetFontColor(Color.black)
	worldTwo.visible = true


# Load third world and set font color.
func LoadWorldThree():
	SetFontColor(Color.white)
	worldThree.visible = true


# Load fourth world and set font color.
func LoadWorldFour():
	SetFontColor(Color.white)
	worldFour.visible = true


# Load fifth world and set font color.
func LoadWorldFive():
	SetFontColor(Color.white)
	worldFive.visible = true


# Set the font color.
func SetFontColor(color):
	$Score.add_color_override("font_color", color)
	$CollectMessage.add_color_override("font_color", color)
	$LifePoints.add_color_override("font_color", color)


# Reset everything at game over.
func ResetWorldsAtGameOver():
	worldOne.visible = false
	worldTwo.visible = false
	worldThree.visible = false
	worldFour.visible = false
	worldFive.visible = false
	
	player.touch_buttons.get_child(0).hide()
	player.touch_buttons.get_child(1).hide()
	player.touch_buttons.get_child(2).hide()
	player.touch_buttons.get_child(3).hide()


# Open the screen where you can choose a character.
func OpenCharacterChooser():
	characterChooser = load("res://Scenes/CharacterChooser.tscn").instance()
	characterChooser.world = self
	get_parent().add_child(characterChooser)
	
	# hide touch buttons
	player.touch_buttons.get_child(0).hide()
	player.touch_buttons.get_child(1).hide()
	player.touch_buttons.get_child(2).hide()
	player.touch_buttons.get_child(3).hide()


# Load the boy as character.
func LoadBoyCharacter():
	boyLifePointOne.visible = true
	boyLifePointTwo.visible = true
	boyLifePointThree.visible = true
	
	player.animation_boy.visible = true


# Load the girl as character.
func LoadGirlCharacter():
	girlLifePointOne.visible = true
	girlLifePointTwo.visible = true
	girlLifePointThree.visible = true
	
	player.animation_girl.visible = true




